# authentic2 - versatile identity manager
# Copyright (C) 2022 Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django import forms
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _

from authentic2.forms.mixins import SlugMixin
from authentic2.utils.evaluate import ExpressionError, validate_condition

from .models import BaseAuthenticator, LoginPasswordAuthenticator


class AuthenticatorFormMixin:
    def clean_show_condition(self):
        condition = self.cleaned_data['show_condition']
        if condition:
            try:
                validate_condition(condition)
            except ExpressionError as e:
                raise ValidationError(e.message)
        return condition


class AuthenticatorAddForm(SlugMixin, forms.ModelForm):
    field_order = ('authenticator', 'name', 'ou')

    authenticator = forms.ChoiceField(label=_('Authenticator'))

    class Meta:
        model = BaseAuthenticator
        fields = ('name',)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.authenticators = {
            x.type: x for x in BaseAuthenticator.__subclasses__() if not x.unique or x.objects.count() < 1
        }
        self.fields['authenticator'].choices = [
            (k, v._meta.verbose_name) for k, v in self.authenticators.items()
        ]

    def save(self):
        Authenticator = self.authenticators[self.cleaned_data['authenticator']]
        self.instance = Authenticator(name=self.cleaned_data['name'])
        return super().save()


class LoginPasswordAuthenticatorEditForm(AuthenticatorFormMixin, forms.ModelForm):
    class Meta:
        model = LoginPasswordAuthenticator
        exclude = ('name', 'slug', 'ou')
